
module Gorillapi
  module PrintResponse
    def print_request(builder, print_only)

      case print_only
      when 'curl'
        show_curl_request(builder)
      when 'details'
        request_details
      else
        request_details
        show_curl_request(builder)
      end

      true
    end

    def request_details
      puts
      puts "-" * 100
      puts
      puts "Request Details: #{@caller}"
      puts "-" * 100
      puts "Url:          #{url}"
      puts "Request:      #{@request_method}"
      puts "Endpoint:     #{@path}"
      puts "Content-Type  #{content_type}"
      puts "Accept:       #{accept_header}"
      puts "Api Token:    #{token}"
      puts "Params:       #{@params}"
      puts
    end

    def print_results(builder, results)
      request_details
      show_curl_request(builder)

      puts
      puts "Api Call Results:"
      puts "-" * 100
      puts results
      puts
    end

    def show_curl_request(builder)
      puts
      puts "cURL Request:"
      puts "-" * 100
      puts builder.print
      puts
    end
  end

  module RequestMethods
    include PrintResponse

    def handle_print(builder, print_only = false)
      if print_only
        print_request(builder, print_only)
      else
        results = builder.send
        print_results(builder, results)
      end
    end


    def get(path, print_only = false, params: {})
      @caller           = caller[0][/`.*'/][1..-2].humanize.titleize
      @request_method   = 'GET'
      @path             = path
      @params           = params

      builder           = build_request
      handle_print(builder, print_only)
    end


    def post(path, print_only = false, params: {})
      @caller           = caller[0][/`.*'/][1..-2].humanize.titleize
      @request_method   = 'POST'
      @path             = path
      @params           = params

      builder           = build_request
      handle_print(builder, print_only)
    end


    def put(path, print_only = false, params: {})
      @caller           = caller[0][/`.*'/][1..-2].humanize.titleize
      @request_method   = 'PUT'
      @path             = path
      @params           = params

      builder           = build_request
      handle_print(builder, print_only)
    end

    def delete(path, print_only = false, params: {})
      @caller           = caller[0][/`.*'/][1..-2].humanize.titleize
      @request_method   = 'DELETE'
      @path             = path
      @params           = params

      builder           = build_request
      handle_print(builder, print_only)
    end

    def build_request
      options = {
        request_method: @request_method,
        url:            url,
        path:           @path,
        params:         @params,
        content_type:   content_type,
        accept_header:  accept_header,
        token:          token,
      }

      RequestBuilder.new(options)
    end
  end
end
